@@ -205,7 +205,7 @@ EFFlatBuildOneLevel(def, flags)
     if (usecount > 0)
 	efHierSrUses(&efFlatContext, efFlatNodesDeviceless, (ClientData)&usecount);
 
-    if ((usecount == 0) && (efFlatRootUse.use_def->def_devs == NULL))
+    if ((usecount == 0) && (HashGetNumEntries(&efFlatRootUse.use_def->def_devs) == 0))
 	efFlatRootUse.use_def->def_flags |= DEF_NODEVICES;
 
     /* Record all local nodes */
@@ -372,7 +372,7 @@ efFlatNodesDeviceless(hc, cdata)
     if (newcount > 0)
 	efHierSrUses(hc, efFlatNodesDeviceless, (ClientData)&newcount);
 
-    if ((hc->hc_use->use_def->def_devs == NULL) && (newcount == 0))
+    if ((HashGetNumEntries(&hc->hc_use->use_def->def_devs) == 0) && (newcount == 0))
     {
 	/* Add all our own nodes to the table */
 	efAddNodes(hc, TRUE);
@@ -527,6 +527,7 @@ efAddNodes(hc, stdcell)
 	    HashSetValue(he, (char *) newname);
 	    newname->efnn_node = newnode;
 	    newname->efnn_hier = hierName;
+	    newname->efnn_port = -1;
 	    if (newnode->efnode_name)
 	    {
 		newname->efnn_next = newnode->efnode_name->efnn_next;
@@ -1018,14 +1019,23 @@ efFlatSingleCap(hc, name1, name2, conn)
     EFNode *n1, *n2;
     HashEntry *he;
     EFCoupleKey ck;
+    static char msg0[] = "cap(1)";
+    static char msg1[] = "cap(2)";
+    char *msg;
+    
+    /* Connections that are below threshold (ext2spice hierarchy only)	*/
+    /* will be missing.  Do not generate errors for these.		*/
 
-    if ((he = EFHNLook(hc->hc_hierName, name1, "cap(1)")) == NULL)
+    msg = (fabs((double)conn->conn_cap / 1000) < EFCapThreshold) ? NULL : msg0;
+
+    if ((he = EFHNLook(hc->hc_hierName, name1, msg)) == NULL)
 	return 0;
     n1 = ((EFNodeName *) HashGetValue(he))->efnn_node;
     if (n1->efnode_flags & EF_KILLED)
 	return 0;
 
-    if ((he = EFHNLook(hc->hc_hierName, name2, "cap(2)")) == NULL)
+    if (msg) msg = msg1;
+    if ((he = EFHNLook(hc->hc_hierName, name2, msg)) == NULL)
 	return 0;
     n2 = ((EFNodeName *) HashGetValue(he))->efnn_node;
     if (n2->efnode_flags & EF_KILLED)
